from __future__ import print_function
import httplib2
import os

from apiclient import discovery
import oauth2client
from oauth2client import client
from oauth2client import tools

import requests

from datetime import datetime

try:
    import argparse
    flags = argparse.ArgumentParser(parents=[tools.argparser]).parse_args()
except ImportError:
    flags = None

# If modifying these scopes, delete your previously saved credentials
# at ~/.credentials/sheets.googleapis.com-python-quickstart.json
SCOPES = 'https://www.googleapis.com/auth/spreadsheets'
CLIENT_SECRET_FILE = 'client_secret.json'
APPLICATION_NAME = 'Google Sheets DHT22'

def get_credentials():
    """Gets valid user credentials from storage.

    If nothing has been stored, or if the stored credentials are invalid,
    the OAuth2 flow is completed to obtain the new credentials.

    Returns:
        Credentials, the obtained credential.
    """
    home_dir = os.path.expanduser('~')
    credential_dir = os.path.join(home_dir, '.credentials')
    if not os.path.exists(credential_dir):
        os.makedirs(credential_dir)
    credential_path = os.path.join(credential_dir,
                                   'sheets.googleapis.com.json')

    store = oauth2client.file.Storage(credential_path)
    credentials = store.get()
    if not credentials or credentials.invalid:
        flow = client.flow_from_clientsecrets(CLIENT_SECRET_FILE, SCOPES)
        flow.user_agent = APPLICATION_NAME
        if flags:
            credentials = tools.run_flow(flow, store, flags)
        else: # Needed only for compatibility with Python 2.6
            credentials = tools.run(flow, store)
        print('Storing credentials to ' + credential_path)
    return credentials

def get_sensor_readings():
    """Get the DHT22 sensor readings from the REST API.

    Get and parse the JSON from the REST API with the sensor readings,
    if something goes wrong, then raises an exception.

    Returns: Data, the parsed JSON as a dictionary object.
    """
    # Consume the REST API for the sensor readings
    response = requests.get('http://127.0.0.1/arduino/readsensor', auth=('<user>', '<pass>'))
    if response.status_code != 200:
        # This means something went wrong.
        raise Exception('GET /arduino/readsensor/ {}'.format(response.status_code))
    data = response.json()
    if data['success'] == False:
        raise Exception('Reading the sensor has failed')
    return data

def main():
    """Append fake data to the DHT22 sheet

    Creates a Sheets API service object and appends some fake data for testing
    DHT22 spreadsheet:
    https://docs.google.com/spreadsheets/d/<ID>/edit
    """
    credentials = get_credentials()
    http = credentials.authorize(httplib2.Http())
    discoveryUrl = ('https://sheets.googleapis.com/$discovery/rest?'
                    'version=v4')
    service = discovery.build('sheets', 'v4', http=http,
                              discoveryServiceUrl=discoveryUrl)

    spreadsheetId = '<ID>'
    rangeName = 'DHT22Readings!A2:C'

    data = get_sensor_readings()
    values = [
        [data['celsius'], data['humidity'], datetime.now().isoformat()]
    ]

    # Upload the values
    body = {
        'values': values
    }
    result = service.spreadsheets().values().append(
        spreadsheetId=spreadsheetId,
        range=rangeName,
        valueInputOption='USER_ENTERED',
        insertDataOption='INSERT_ROWS',
        body=body).execute()

if __name__ == '__main__':
    main()
